# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import components

import pykolab

from pykolab import utils
from pykolab.constants import *
from pykolab.setup.services import *
from pykolab.translate import _

log = pykolab.getLogger('pykolab.setup')
conf = pykolab.getConf()

def __init__():
    components.register(
            'imap',
            execute,
            description=description(),
            after=['ldap']
        )

def description():
    return _("Setup IMAP.")

def execute(*args, **kw):
    """
        Apply the necessary settings to /etc/imapd.conf
    """

    imapd_settings = {
            "ldap_servers": conf.get('ldap', 'ldap_uri'),
            "ldap_base": conf.get('ldap', 'base_dn'),
            "ldap_bind_dn": conf.get('ldap', 'service_bind_dn'),
            "ldap_password": conf.get('ldap', 'service_bind_pw'),
            "ldap_filter": '(|(&(|(uid=%s)(uid=cyrus-murder))(uid=%%U))(&(|(uid=%%U)(mail=%%U@%%d)(mail=%%U@%%r))(objectclass=kolabinetorgperson)))' % (conf.get('cyrus-imap', 'admin_login')),
            "ldap_user_attribute": conf.get('cyrus-sasl', 'result_attribute'),
            "ldap_group_base": conf.get('ldap', 'base_dn'),
            "ldap_group_filter": "(&(cn=%u)(objectclass=ldapsubentry)(objectclass=nsroledefinition))",
            "ldap_group_scope": "one",
            "ldap_member_base": conf.get('ldap','user_base_dn'),
            "ldap_member_method": "attribute",
            "ldap_member_attribute": "nsrole",
            "admins": conf.get('cyrus-imap', 'admin_login'),
            "postuser": "shared",
        }

    # Apply the settings.

    matching_config = True

    for template_basename, output_file, settings in [
        ('imapd.conf.tpl', '/etc/imapd.conf', [imapd_settings]),
        ('cyrus.conf.tpl', '/etc/cyrus.conf', [{}])
        ]:

        template_file = get_template_path(template_basename)

        # Instantiate the template unless only checking is being performed.

        if template_file is not None:
            matching_config = matching_config and instantiate_template(template_file, output_file, settings, check_only=conf.check_only)
        else:
            log.error(_("Could not write out Cyrus IMAP configuration file %s") % output_file)
            return

    # When only checking, test the annotations file, give the status and return.

    if conf.check_only:
        matching_config = matching_config and file_contains_data('/etc/imapd.annotations.conf', annotations, exact=True)
        utils.setup_status("imap", matching_config and _("setup done") or _("needs setup"))
        return

    # Write the annotations file.

    fp = open('/etc/imapd.annotations.conf', 'w')
    fp.write(annotations)
    fp.close()

    set_service_default('kolab-saslauthd', 'START', 'yes')

    if not (
        control_service('saslauthd', 'stop') and
        control_service('kolab-saslauthd', 'restart') and
        control_service('cyrus-imapd', 'restart')
        ):

        log.error(_("Could not start the cyrus-imapd and kolab-saslauthd services."))

    if not (
        configure_service('saslauthd', False) and
        configure_service('kolab-saslauthd', True) and
        configure_service('cyrus-imapd', True)
        ):

        log.error(_("Could not configure to start on boot, the " + \
                "cyrus-imapd and kolab-saslauthd services."))

# Data used above.

annotations = """\
/vendor/kolab/activesync,mailbox,string,backend,value.priv,r
/vendor/kolab/color,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/displayname,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/folder-test,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/folder-type,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/incidences-for,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/pxfb-readable-for,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/uniqueid,mailbox,string,backend,value.shared value.priv,a
/vendor/kolab/h-share-attr-desc,mailbox,string,backend,value.shared value.priv,a
/vendor/horde/share-params,mailbox,string,backend,value.shared value.priv,a
/vendor/x-toltec/test,mailbox,string,backend,value.shared value.priv,a
"""
